package com.sunyard.security.util;

import java.util.Arrays;

public class ByteUtil {
    /**
     * PKCS5/PKCS7 标准填充数据
     * @param data
     * @return
     */
    public static byte[] pkcsFullPadding(byte[] data) {
        int remainder = data.length % 16;
        remainder = 16 - remainder;

        byte[] buff = new byte[ data.length + remainder ];
        System.arraycopy( data, 0, buff, 0, data.length );
        Arrays.fill( buff, data.length, buff.length, (byte)( 0xFF & remainder ));
        return buff;
    }



    /**
     * PKCS5/PKCS7 标准去除填充数据
     * @param data
     * @param len 有效数据长度
     * @return
     */
    public static byte[] pkcsRemovePadding(byte[] data, int len) {
        if ( data.length < len ) {
            throw new IllegalArgumentException();
        }
        int remainder = data[ len - 1 ];
        if ( remainder < 0 || remainder > 16 || remainder > len ) {
            throw new IllegalArgumentException();
        }
        len -= remainder;
        byte[] buff = new byte[ len ];
        System.arraycopy( data, 0, buff, 0, len );
        return buff;
    }

    public static byte[] pkcsRemovePadding(byte[] data){
        return pkcsRemovePadding( data, data.length );
    }


    public static String bytes2HexString(byte[] b) {
        StringBuilder bs = new StringBuilder();

        for (int i = 0; i < b.length; ++i) {
            String hex = Integer.toHexString(b[i] & 255);
            if (hex.length() == 1) {
                hex = '0' + hex;
            }

            bs.append(hex.toUpperCase());
        }

        return bs.toString();
    }

    public static String bytes2HexString(byte[] b, int len) {
        StringBuilder bs = new StringBuilder();

        for (int i = 0; i < len; ++i) {
            String hex = Integer.toHexString(b[i] & 255);
            if (hex.length() == 1) {
                hex = '0' + hex;
            }

            bs.append(hex.toUpperCase());
        }

        return bs.toString();
    }

    private static byte charToByte(char c) {
        return (byte) "0123456789ABCDEF".indexOf(c);
    }

    public static byte[] hexString2Bytes(String hex) {
        if (hex != null && !hex.equals("")) {
            if (hex.length() % 2 != 0) {
                return new byte[0];
            } else {
                hex = hex.toUpperCase();
                int len = hex.length() / 2;
                byte[] b = new byte[len];
                char[] hc = hex.toCharArray();

                for (int i = 0; i < len; ++i) {
                    int p = 2 * i;
                    b[i] = (byte) (charToByte(hc[p]) << 4 | charToByte(hc[p + 1]) & 0xff );
                }

                return b;
            }
        } else {
            return new byte[0];
        }
    }


    public static byte[] safeStringBytes(String str){
        if ( null == str || str.isEmpty()) {
            return new byte[] { 0x30 };
        }
        int len = str.getBytes().length;
        byte[] ret = new byte[len + 1];
        System.arraycopy(str.getBytes(), 0, ret, 0, len);
        return ret;
    }
}
