import com.sun.jna.Pointer;
import com.sun.jna.ptr.IntByReference;
import com.sun.jna.ptr.PointerByReference;
import com.sunyard.security.sdf.AlgorithmEnum;
import com.sunyard.security.sdf.SDFJna;
import com.sunyard.security.util.ByteUtil;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class SDFTest {

    private  PointerByReference phDeviceHandle = new PointerByReference();
    private  PointerByReference phSessionHandle = new PointerByReference();;
    private static final byte[] data = new byte[16];
    private static final byte[] symKey = new byte[]{9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9};
    private static final String ip = "172.16.17.245";


    @Before
    public void before(){
        // 打开设备
        int ret = SDFJna.INSTANCE.SDF_OpenDevice(
                phDeviceHandle,
                ByteUtil.safeStringBytes(ip),
                8889,
                3000,
                3000,
                0);

        if (ret != 0) {
            System.err.println("打开设备失败 错误码：" + String.format("0x%02X", ret));
            throw new RuntimeException(String.format("0x%02X", ret));
        }
        Pointer hDeviceHandle = phDeviceHandle.getValue();
        // 打开会话
        ret = SDFJna.INSTANCE.SDF_OpenSession(hDeviceHandle, phSessionHandle);
        if (ret != 0) {
            System.err.println("打开设备失败 错误码：" + String.format("0x%02X", ret));
            throw new RuntimeException(String.format("0x%02X", ret));
        }
    }


    @After
    public void after(){
        if ( null !=  phSessionHandle.getValue() ) {
            SDFJna.INSTANCE.SDF_CloseSession( phSessionHandle.getValue() );
        }
        if ( null != phDeviceHandle.getValue() ) {
            SDFJna.INSTANCE.SDF_CloseDevice(phDeviceHandle.getValue());
        }
    }

    @Test
    public void random(){

        int oneSize = 16;
        byte[] oneRadom = new byte[16]; // 注意这里初始化大小应为 oneSize
        int ret = SDFJna.INSTANCE.SDF_GenerateRandom( phSessionHandle.getValue(), oneSize, oneRadom );

        if (ret != 0) {
            throw new RuntimeException(String.format("0x%02X", ret));
        } else {
            System.out.println("调用成功");
        }
        Assert.assertEquals( ret, 0 );
        System.out.println( ByteUtil.bytes2HexString( oneRadom ) );
    }


    private void genSessionKey(PointerByReference phkeyHandle){

        int ret = SDFJna.INSTANCE.SDF_ImportKey(phSessionHandle.getValue(), symKey, symKey.length ,phkeyHandle);
        if (ret != 0) {
            throw new RuntimeException("genSessionKey " + String.format("0x%02X", ret));
        }
    }


    /**
     * SDF 接口内没有填充
     * 输入数据必须 16 的整倍数
     */
    @Test
    public void sym_ecb(){
        int ret = 0;
        PointerByReference phkeyHandle = new PointerByReference();
        genSessionKey(phkeyHandle);


        System.out.println( "data=" + ByteUtil.bytes2HexString( data ));
        byte[] enData = new byte[32];
        IntByReference intByReference = new IntByReference();
        ret = SDFJna.INSTANCE.SDF_Encrypt(phSessionHandle.getValue(), phkeyHandle.getValue(), AlgorithmEnum.SGD_SM4_ECB.getValue(), null, data, data.length, enData, intByReference);
        if (ret != 0) {
            throw new RuntimeException("加密失败 " + String.format("0x%02X", ret));
        }
        System.out.println( intByReference.getValue() );
        System.out.println( "enData=" + ByteUtil.bytes2HexString( enData ));
        byte[] rawData = new byte[32];
        ret = SDFJna.INSTANCE.SDF_Decrypt(phSessionHandle.getValue(), phkeyHandle.getValue(), AlgorithmEnum.SGD_SM4_ECB.getValue(), null, enData, intByReference.getValue(), rawData, intByReference);
        if (ret != 0) {
            throw new RuntimeException("解密失败 " + String.format("0x%02X", ret));
        }
        System.out.println("长度" + intByReference.getValue());
        System.out.println( "rawData=" + ByteUtil.bytes2HexString( rawData , intByReference.getValue()));
    }


    @Test
    public void sym_cbc(){
        int ret = 0;
        PointerByReference phkeyHandle = new PointerByReference();
        genSessionKey(phkeyHandle);

        byte[] iv = new byte[16];

        System.out.println( "data=" + ByteUtil.bytes2HexString( data ));
        byte[] enData = new byte[32];
        IntByReference intByReference = new IntByReference();
        ret = SDFJna.INSTANCE.SDF_Encrypt(phSessionHandle.getValue(), phkeyHandle.getValue(), AlgorithmEnum.SGD_SM4_CBC.getValue(), iv, data, data.length, enData, intByReference);
        if (ret != 0) {
            throw new RuntimeException("加密失败 " + String.format("0x%02X", ret));
        }
        System.out.println( intByReference.getValue() );
        System.out.println( "enData=" + ByteUtil.bytes2HexString( enData ));
        byte[] rawData = new byte[32];
        iv = new byte[16];
        ret = SDFJna.INSTANCE.SDF_Decrypt(phSessionHandle.getValue(), phkeyHandle.getValue(), AlgorithmEnum.SGD_SM4_CBC.getValue(), iv, enData, intByReference.getValue(), rawData, intByReference);
        if (ret != 0) {
            throw new RuntimeException("解密失败 " + String.format("0x%02X", ret));
        }
        System.out.println( "rawData=" + ByteUtil.bytes2HexString( rawData , intByReference.getValue()));
    }

    @Test
    public void mac(){
        int ret = 0;
        PointerByReference phkeyHandle = new PointerByReference();
        genSessionKey(phkeyHandle);

        byte[] mac = new byte[32];
        IntByReference macLen = new IntByReference();
        ret = SDFJna.INSTANCE.SDF_CalculateMAC(phSessionHandle.getValue(),
                phkeyHandle.getValue(),
                AlgorithmEnum.SGD_SM4_MAC.getValue(),
                new byte[16],
                data,
                data.length,
                mac,
                macLen
        );
        if (ret != 0) {
            throw new RuntimeException("CalculateMAC " + String.format("0x%02X", ret));
        }
        System.out.println( ByteUtil.bytes2HexString( mac , macLen.getValue()) );
    }


    @Test
    public void hash(){
        int ret = 0;
        byte[] mac = new byte[32];
        IntByReference macLen = new IntByReference();

        ret = SDFJna.INSTANCE.SDF_HashInit( phSessionHandle.getValue(), AlgorithmEnum.SGD_SM3.getValue(), null, null, 0 );
        if (ret != 0) {
            throw new RuntimeException("HashInit " + String.format("0x%02X", ret));
        }
        ret = SDFJna.INSTANCE.SDF_HashUpdate(phSessionHandle.getValue(), data, data.length);
        if (ret != 0) {
            throw new RuntimeException("HashUpdate " + String.format("0x%02X", ret));
        }
        ret = SDFJna.INSTANCE.SDF_HashFinal(phSessionHandle.getValue(), mac, macLen);
        if (ret != 0) {
            throw new RuntimeException("HashFinal " + String.format("0x%02X", ret));
        }
        System.out.println( ByteUtil.bytes2HexString( mac , macLen.getValue()) );
    }


    /**
     * 非对称 签名验签
     * 外部密钥
     */
    @Test
    public void asym_ext_sv(){
        byte[] puk = new byte[256];
        byte[] pak = new byte[256];
        byte[] hash = new byte[32];
        int ret = SDFJna.INSTANCE.SDF_GenerateKeyPair_ECC(phSessionHandle.getValue(), AlgorithmEnum.SGD_SM2.getValue(), 256, puk, pak);
        if (ret != 0) {
            throw new RuntimeException("生成非对称密钥 " + String.format("0x%02X", ret));
        }

        byte[] sign = new byte[512];
        ret = SDFJna.INSTANCE.SDF_ExternalSign_ECC(
                    phSessionHandle.getValue(),
                    AlgorithmEnum.SGD_SM2.getValue(), pak, hash, hash.length, sign
                );
        if (ret != 0) {
            throw new RuntimeException("ExternalSign_ECC " + String.format("0x%02X", ret));
        }

        // 验签
        ret = SDFJna.INSTANCE.SDF_ExternalVerify_ECC(phSessionHandle.getValue(), AlgorithmEnum.SGD_SM2.getValue(), puk,  hash, hash.length, sign);
        if (ret != 0) {
            throw new RuntimeException("ExternalSign_ECC " + String.format("0x%02X", ret));
        }
    }


    /**
     * 非对称 加密解密
     * 外部密钥
     */
    @Test
    public void asym_en(){

        byte[] puk = new byte[256];
        byte[] pak = new byte[256];
        int ret =  SDFJna.INSTANCE.SDF_GenerateKeyPair_ECC(phSessionHandle.getValue(), AlgorithmEnum.SGD_SM2.getValue(), 256, puk, pak);
        if (ret != 0) {
            throw new RuntimeException("生成非对称密钥 " + String.format("0x%02X", ret));
        }


        System.out.println( "ret= " + ret);
        System.out.println( "puk= " + ByteUtil.bytes2HexString( puk ));
        System.out.println( "pak= " + ByteUtil.bytes2HexString( pak ));



        byte[] enc = new byte[512];
        ret =  SDFJna.INSTANCE.SDF_ExternalEncrypt_ECC(
                phSessionHandle.getValue(),
                AlgorithmEnum.SGD_SM2.getValue(),
                puk,
                data, data.length, enc
        );
        if (ret != 0) {
            throw new RuntimeException("ExternalEncrypt_ECC " + String.format("0x%02X", ret));
        }

        byte[] buffer = new byte[512];
        IntByReference bufferLen = new IntByReference( buffer.length );
        ret =  SDFJna.INSTANCE.SDF_ExternalDecrypt_ECC(
                phSessionHandle.getValue(),
                AlgorithmEnum.SGD_SM2.getValue(),
                pak,
                enc, buffer, bufferLen
        );
        if (ret != 0) {
            throw new RuntimeException("ExternalEncrypt_ECC " + String.format("0x%02X", ret));
        }
    }
}